==> frontend_editing-1.1.0.tar.gz: frontend_editing/src/ParagraphsHelper.php <==
<?php

namespace Drupal\frontend_editing;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Url;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs_edit\ParagraphLineageInspector;
use Drupal\paragraphs_edit\ParagraphLineageRevisioner;

/**
 * Class ParagraphsHelper contains methods to manage paragraphs crud operations.
 *
 * @package frontend_editing
 */
class ParagraphsHelper implements ParagraphsHelperInterface {

  /**
   * The lineage inspector.
   *
   * @var \Drupal\paragraphs_edit\ParagraphLineageInspector
   */
  protected $lineageInspector;

  /**
   * The lineage revisioner.
   *
   * @var \Drupal\paragraphs_edit\ParagraphLineageRevisioner
   */
  protected $lineageRevisioner;

  /**
   * ParagraphsHelper constructor.
   *
   * @param \Drupal\paragraphs_edit\ParagraphLineageInspector $paragraph_lineage_inspector
   *   The lineage inspector.
   * @param \Drupal\paragraphs_edit\ParagraphLineageRevisioner $paragraph_lineage_revisioner
   *   The lineage revisioner.
   */
  public function __construct(ParagraphLineageInspector $paragraph_lineage_inspector, ParagraphLineageRevisioner $paragraph_lineage_revisioner) {
    $this->lineageInspector = $paragraph_lineage_inspector;
    $this->lineageRevisioner = $paragraph_lineage_revisioner;
  }

  /**
   * {@inheritdoc}
   */
  public function allowUp(ParagraphInterface $paragraph) {
    return $this->allow($paragraph, 'up');
  }

  /**
   * {@inheritdoc}
   */
  public function allowDown(ParagraphInterface $paragraph) {
    return $this->allow($paragraph, 'down');
  }

  /**
   * Checks if the paragraph can be moved up or down.
   *
   * @param \Drupal\paragraphs\ParagraphInterface $paragraph
   *   The paragraph to check.
   * @param string $operation
   *   The operation to check.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  protected function allow(ParagraphInterface $paragraph, $operation) {
    // Check that the operation is valid.
    if (!in_array($operation, ['up', 'down'])) {
      return AccessResult::forbidden();
    }
    // Get paragraph parent entity.
    $parent = $paragraph->getParentEntity();
    // Check that the parent entity exists and the user has update access.
    if (!$parent || !$parent->access('update')) {
      return AccessResult::forbidden();
    }
    // Check that the parent entity has the paragraph field.
    $parent_field_name = $paragraph->get('parent_field_name')->value;
    if (!$parent->hasField($parent_field_name) || $parent->get($parent_field_name)->isEmpty()) {
      return AccessResult::forbidden();
    }
    // Get the paragraph items.
    $paragraph_items = $parent->get($parent_field_name)->getValue();
    if ($operation == 'up') {
      $item = reset($paragraph_items);
    }
    else {
      $item = end($paragraph_items);
    }
    if ($item['target_id'] == $paragraph->id()) {
      return AccessResult::forbidden();
    }
    else {
      return AccessResult::allowed();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function move(ParagraphInterface $paragraph, $operation) {
    $allow = $this->allow($paragraph, $operation);
    if (!$allow->isAllowed()) {
      return FALSE;
    }
    $parent = $paragraph->getParentEntity();
    $parent_field_name = $paragraph->get('parent_field_name')->value;
    $paragraph_items = $parent->get($parent_field_name)->getValue();
    if ($operation == 'up') {
      foreach ($paragraph_items as $delta => $paragraph_item) {
        if ($paragraph_item['target_id'] == $paragraph->id()) {
          if ($delta > 0) {
            $prev_paragraph = $paragraph_items[$delta - 1];
            $paragraph_items[$delta - 1] = $paragraph_items[$delta];
            $paragraph_items[$delta] = $prev_paragraph;
          }
          break;
        }
      }
    }
    else {
      $numitems = count($paragraph_items);
      foreach ($paragraph_items as $delta => $paragraph_item) {
        if ($paragraph_item['target_id'] == $paragraph->id()) {
          if ($delta < $numitems) {
            $next_paragraph = $paragraph_items[$delta + 1];
            $paragraph_items[$delta + 1] = $paragraph_items[$delta];
            $paragraph_items[$delta] = $next_paragraph;
          }
          break;
        }
      }
    }
    $parent->get($parent_field_name)->setValue($paragraph_items);
    $root_parent = $this->lineageInspector->getRootParent($paragraph);
    if ($this->lineageRevisioner->shouldCreateNewRevision($root_parent)) {
      $this->lineageRevisioner->saveNewRevision($parent);
    }
    else {
      $parent->save();
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(ParagraphInterface $paragraph) {
    $entity = $this->lineageInspector->getRootParent($paragraph);
    return $entity->hasLinkTemplate('canonical') ? $entity->toUrl() : Url::fromRoute('<front>');
  }

  /**
   * {@inheritdoc}
   */
  public function getParagraphRootParent(ParagraphInterface $paragraph) {
    return $this->lineageInspector->getRootParent($paragraph);
  }

}

==> frontend_editing-1.2.5.tar.gz: frontend_editing/src/ParagraphsHelper.php <==
<?php

namespace Drupal\frontend_editing;

use Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException;
use Drupal\Component\Plugin\Exception\PluginNotFoundException;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Url;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs\ParagraphsTypeInterface;
use Drupal\paragraphs_edit\ParagraphLineageInspector;
use Drupal\paragraphs_edit\ParagraphLineageRevisioner;

/**
 * Class ParagraphsHelper contains methods to manage paragraphs crud operations.
 *
 * @package frontend_editing
 */
class ParagraphsHelper implements ParagraphsHelperInterface {

  /**
   * The lineage inspector.
   *
   * @var \Drupal\paragraphs_edit\ParagraphLineageInspector
   */
  protected $lineageInspector;

  /**
   * The lineage revisioner.
   *
   * @var \Drupal\paragraphs_edit\ParagraphLineageRevisioner
   */
  protected $lineageRevisioner;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * ParagraphsHelper constructor.
   *
   * @param \Drupal\paragraphs_edit\ParagraphLineageInspector $paragraph_lineage_inspector
   *   The lineage inspector.
   * @param \Drupal\paragraphs_edit\ParagraphLineageRevisioner $paragraph_lineage_revisioner
   *   The lineage revisioner.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(ParagraphLineageInspector $paragraph_lineage_inspector, ParagraphLineageRevisioner $paragraph_lineage_revisioner, EntityTypeManagerInterface $entity_type_manager) {
    $this->lineageInspector = $paragraph_lineage_inspector;
    $this->lineageRevisioner = $paragraph_lineage_revisioner;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function allowUp(ParagraphInterface $paragraph) {
    return $this->allow($paragraph, 'up');
  }

  /**
   * {@inheritdoc}
   */
  public function allowDown(ParagraphInterface $paragraph) {
    return $this->allow($paragraph, 'down');
  }

  /**
   * {@inheritdoc}
   */
  public function allowAdd($parent_type, $parent, $parent_field_name) {
    try {
      $parent_entity = $this->entityTypeManager->getStorage($parent_type)
        ->load($parent);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      return AccessResult::forbidden();
    }
    if (!$parent_entity || !$parent_entity->access('update') || !$parent_entity->hasField($parent_field_name)) {
      return AccessResult::forbidden();
    }
    // Check that the given paragraphs type is allowed.
    $parent_field_definition = $parent_entity->get($parent_field_name)->getFieldDefinition();
    $cardinality = $parent_field_definition->getFieldStorageDefinition()->getCardinality();
    if ($cardinality > 0 && $parent_entity->get($parent_field_name)->count() == $cardinality) {
      return AccessResult::forbidden();
    }
    return AccessResult::allowed();
  }

  /**
   * {@inheritdoc}
   */
  public function allowAddType(ParagraphsTypeInterface $paragraphs_type, $parent_type, $parent, $parent_field_name) {
    // Check that the parent entity exists and the user has update access.
    try {
      $parent_entity = $this->entityTypeManager->getStorage($parent_type)
        ->load($parent);
    }
    catch (InvalidPluginDefinitionException | PluginNotFoundException $e) {
      return AccessResult::forbidden();
    }
    if (!$parent_entity || !$parent_entity->access('update') || !$parent_entity->hasField($parent_field_name)) {
      return AccessResult::forbidden();
    }
    // Check that the given paragraphs type is allowed.
    $parent_field_definition = $parent_entity->get($parent_field_name)->getFieldDefinition();
    $handler_settings = $parent_field_definition->getSetting('handler_settings');
    if (!empty($handler_settings['target_bundles']) && !in_array($paragraphs_type->id(), $handler_settings['target_bundles'])) {
      return AccessResult::forbidden();
    }
    $cardinality = $parent_field_definition->getFieldStorageDefinition()->getCardinality();
    if ($cardinality > 0 && $parent_entity->get($parent_field_name)->count() == $cardinality) {
      return AccessResult::forbidden();
    }
    return AccessResult::allowed();
  }

  /**
   * Checks if the paragraph can be moved up or down.
   *
   * @param \Drupal\paragraphs\ParagraphInterface $paragraph
   *   The paragraph to check.
   * @param string $operation
   *   The operation to check.
   *
   * @return \Drupal\Core\Access\AccessResult
   *   The access result.
   */
  protected function allow(ParagraphInterface $paragraph, $operation) {
    // Check that the operation is valid.
    if (!in_array($operation, ['up', 'down'])) {
      return AccessResult::forbidden();
    }
    if ($paragraph->isNew()) {
      return AccessResult::forbidden();
    }
    // Get paragraph parent entity.
    $parent = $paragraph->getParentEntity();
    // Check that the parent entity exists and the user has update access.
    if (!$parent || !$parent->access('update')) {
      return AccessResult::forbidden();
    }
    // Check that the parent entity has the paragraph field.
    $parent_field_name = $paragraph->get('parent_field_name')->value;
    if (!$parent->hasField($parent_field_name) || $parent->get($parent_field_name)->isEmpty()) {
      return AccessResult::forbidden();
    }
    // Get the paragraph items.
    $paragraph_items = $parent->get($parent_field_name)->getValue();
    if ($operation == 'up') {
      $item = reset($paragraph_items);
    }
    else {
      $item = end($paragraph_items);
    }
    if ($item['target_id'] == $paragraph->id()) {
      return AccessResult::forbidden();
    }
    else {
      return AccessResult::allowed();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function move(ParagraphInterface $paragraph, $operation) {
    $allow = $this->allow($paragraph, $operation);
    if (!$allow->isAllowed()) {
      return FALSE;
    }
    $parent = $paragraph->getParentEntity();
    $parent_field_name = $paragraph->get('parent_field_name')->value;
    $paragraph_items = $parent->get($parent_field_name)->getValue();
    if ($operation == 'up') {
      foreach ($paragraph_items as $delta => $paragraph_item) {
        if ($paragraph_item['target_id'] == $paragraph->id()) {
          if ($delta > 0) {
            $prev_paragraph = $paragraph_items[$delta - 1];
            $paragraph_items[$delta - 1] = $paragraph_items[$delta];
            $paragraph_items[$delta] = $prev_paragraph;
          }
          break;
        }
      }
    }
    else {
      $numitems = count($paragraph_items);
      foreach ($paragraph_items as $delta => $paragraph_item) {
        if ($paragraph_item['target_id'] == $paragraph->id()) {
          if ($delta < $numitems) {
            $next_paragraph = $paragraph_items[$delta + 1];
            $paragraph_items[$delta + 1] = $paragraph_items[$delta];
            $paragraph_items[$delta] = $next_paragraph;
          }
          break;
        }
      }
    }
    $parent->get($parent_field_name)->setValue($paragraph_items);
    $root_parent = $this->lineageInspector->getRootParent($paragraph);
    if ($this->lineageRevisioner->shouldCreateNewRevision($root_parent)) {
      $this->lineageRevisioner->saveNewRevision($parent);
    }
    else {
      $parent->save();
    }
    return TRUE;
  }

  /**
   * {@inheritdoc}
   */
  public function getRedirectUrl(ParagraphInterface $paragraph) {
    $entity = $this->lineageInspector->getRootParent($paragraph);
    return $entity->hasLinkTemplate('canonical') ? $entity->toUrl() : Url::fromRoute('<front>');
  }

  /**
   * {@inheritdoc}
   */
  public function getParagraphRootParent(ParagraphInterface $paragraph) {
    return $this->lineageInspector->getRootParent($paragraph);
  }

}
